@@ -2,7 +2,7 @@ package Test2::Tools::TypeTiny;
 
 # ABSTRACT: Test2 tools for checking Type::Tiny types
 use version;
-our $VERSION = 'v0.90.1'; # VERSION
+our $VERSION = 'v0.91.0'; # VERSION
 
 use v5.18;
 use strict;
@@ -10,12 +10,13 @@ use warnings;
 
 use parent 'Exporter';
 
-use List::Util v1.29 qw< uniq pairmap pairs >;
+use List::Util v1.29 qw< uniq shuffle pairmap pairs >;
 use Scalar::Util     qw< refaddr >;
 
 use Test2::API            qw< context run_subtest >;
 use Test2::Tools::Basic;
-use Test2::Tools::Compare qw< is >;
+use Test2::Tools::Compare qw< is like >;
+use Test2::Compare        qw< compare strict_convert >;
 
 use Data::Dumper;
 
@@ -25,8 +26,9 @@ use namespace::clean;
 #pod
 #pod =head1 SYNOPSIS
 #pod
-#pod     use Test2::Tools::Basic;
+#pod     use Test2::V0;
 #pod     use Test2::Tools::TypeTiny;
+#pod
 #pod     use MyTypes qw< FullyQualifiedDomainName >;
 #pod
 #pod     type_subtest FullyQualifiedDomainName, sub {
@@ -53,6 +55,15 @@ use namespace::clean;
 #pod                 nonprod3-foobar-me          nonprod3-foobar-me.ourdomain.com
 #pod             >,
 #pod         );
+#pod         should_sort_into(
+#pod             $type,
+#pod             [qw< ftp001-prod3 ftp001-prod3.ourdomain.com prod-ask-me.ourdomain.com >],
+#pod         );
+#pod
+#pod         like $type->get_message(undef), qr<Must be a valid FQDN>, 'error message is correct';
+#pod         like $type->validate_explain(undef), [
+#pod             qr<Undef did not pass type constraint>,
+#pod         ], 'deep explanation is correct';
 #pod     };
 #pod
 #pod     done_testing;
@@ -71,7 +82,8 @@ use namespace::clean;
 
 our @EXPORT_OK = (qw<
     type_subtest
-    should_pass_initially should_fail_initially should_pass should_fail should_coerce_into
+    should_pass_initially should_fail_initially should_pass should_fail
+    should_coerce_into should_sort_into
 >);
 our @EXPORT = @EXPORT_OK;
 
@@ -111,6 +123,10 @@ sub type_subtest ($&) {
 #pod
 #pod These functions are most useful wrapped inside of a L</type_subtest> coderef.
 #pod
+#pod Note that most of these checks will run through C<get_message> and C<validate_explain> calls to
+#pod confirm the coderefs don't die.  If you need to validate the error messages themselves, consider
+#pod using checks similar to the ones in the L</SYNOPSIS>.
+#pod
 #pod =head3 should_pass_initially
 #pod
 #pod     should_pass_initially($type, @values);
@@ -141,6 +157,8 @@ sub _should_pass_initially_subtest {
     foreach my $value (@values) {
         my $val_dd      = _dd($value);
         my @val_explain = _constraint_type_check_debug_map($type, $value);
+        _check_error_message_methods($type, $value);
+
         ok $type->check($value), "$val_dd should pass", @val_explain;
     }
 }
@@ -179,6 +197,8 @@ sub _should_fail_initially_subtest {
     foreach my $value (@values) {
         my $val_dd      = _dd($value);
         my @val_explain = _constraint_type_check_debug_map($type, $value);
+        _check_error_message_methods($type, $value);
+
         ok !$type->check($value), "$val_dd should fail", @val_explain;
     }
 }
@@ -218,6 +238,7 @@ sub _should_pass_subtest {
     foreach my $value (@values) {
         my $val_dd      = _dd($value);
         my @val_explain = _constraint_type_check_debug_map($type, $value);
+        _check_error_message_methods($type, $value);
 
         if ($type->check($value)) {
             pass "$val_dd should pass (initial check)", @val_explain;
@@ -236,6 +257,7 @@ sub _should_pass_subtest {
             fail "$val_dd should pass (failed coercion)", @val_explain, @coercion_debug;
             next;
         }
+        _check_error_message_methods($type, $new_value);
 
         # final check
         @val_explain = _constraint_type_check_debug_map($type, $new_value);
@@ -273,6 +295,7 @@ sub _should_fail_subtest {
     foreach my $value (@values) {
         my $val_dd      = _dd($value);
         my @val_explain = _constraint_type_check_debug_map($type, $value);
+        _check_error_message_methods($type, $value);
 
         if ($type->check($value)) {
             fail "$val_dd should fail (initial check)", @val_explain;
@@ -291,6 +314,7 @@ sub _should_fail_subtest {
             pass "$val_dd should fail (failed coercion)", @val_explain, @coercion_debug;
             next;
         }
+        _check_error_message_methods($type, $new_value);
 
         # final check
         @val_explain = _constraint_type_check_debug_map($type, $new_value);
@@ -335,6 +359,7 @@ sub _should_coerce_into_subtest {
 
         my $val_dd      = _dd($value);
         my @val_explain = _constraint_type_check_debug_map($type, $value);
+        _check_error_message_methods($type, $value);
 
         if ($type->check($value)) {
             fail "$val_dd should fail (initial check)";
@@ -353,6 +378,7 @@ sub _should_coerce_into_subtest {
             fail "$val_dd should coerce", @val_explain, @coercion_debug;
             next;
         }
+        _check_error_message_methods($type, $new_value);
 
         # make sure it matches the expected value
         @val_explain = _constraint_type_check_debug_map($type, $new_value);
@@ -360,6 +386,66 @@ sub _should_coerce_into_subtest {
     }
 }
 
+
+#pod =head3 should_sort_into
+#pod
+#pod     should_sort_into($type, @sorted_arrayrefs);
+#pod
+#pod Creates a L<buffered subtest|Test2::Tools::Subtest/BUFFERED> that confirms the type will sort
+#pod into the expected lists given.  The input list is a shuffled version of the sorted list.
+#pod
+#pod Because this introduces some non-deterministic behavior to the test, it will run through 100 cycles
+#pod of shuffling and sorting to confirm the results.  A good sorter should always return a
+#pod deterministic result for a given list, with enough fallbacks to account for every unique case.
+#pod Any failure will immediate stop the loop and return both the shuffled input and output list in the
+#pod failure output, so that you can temporarily test in a more deterministic manner, as you debug the
+#pod fault.
+#pod
+#pod =cut
+
+sub should_sort_into {
+    my $ctx  = context();
+    my $pass = run_subtest(
+        'should sort into',
+        \&_should_sort_into_subtest,
+        { buffered => 1, inherit_trace => 1 },
+        @_,
+    );
+    $ctx->release;
+
+    return $pass;
+}
+
+sub _should_sort_into_subtest {
+    my ($type, @sorted_lists) = @_;
+
+    plan scalar(@sorted_lists);
+
+    foreach my $sorted_list (@sorted_lists) {
+        my @expected_sort = @$sorted_list;
+
+        my $val_dd = _dd(\@expected_sort);
+
+        my (@shuffled, @sorted);
+        foreach my $i (1..100) {
+            @shuffled = shuffle @expected_sort;
+            @sorted   = $type->sort(@shuffled);
+
+            # To hide all of these iterations, we'll compare with 'compare' first, and if it's a failure,
+            # we'll use 'is' to advertise the failure.
+            my $delta = compare(\@sorted, \@expected_sort, \&strict_convert);
+            last if $delta;  # let 'is' fail
+        }
+
+        # pass or fail
+        my @io_explain = (
+            "Shuffled Input:   "._dd(\@shuffled),
+            "Resulting Output: "._dd(\@sorted),
+        );
+        is \@sorted, \@expected_sort, $val_dd, @io_explain;
+    }
+}
+
 # Helpers
 sub _dd {
     my $dd  = Data::Dumper->new([ shift ])->Terse(1)->Indent(0)->Useqq(1)->Deparse(1)->Quotekeys(0)->Sortkeys(1)->Maxdepth(2);
@@ -434,6 +520,14 @@ sub _check_coercion {
     return $old_value ne $new_value;
 }
 
+sub _check_error_message_methods {
+    my ($type, $value) = @_;
+
+    # If it dies, we just let it naturally die
+    $type->get_message($value);
+    $type->validate_explain($value);  # will return undef on good values
+}
+
 #pod =head1 TROUBLESHOOTING
 #pod
 #pod =head2 Test name output
@@ -521,12 +615,13 @@ Test2::Tools::TypeTiny - Test2 tools for checking Type::Tiny types
 
 =head1 VERSION
 
-version v0.90.1
+version v0.91.0
 
 =head1 SYNOPSIS
 
-    use Test2::Tools::Basic;
+    use Test2::V0;
     use Test2::Tools::TypeTiny;
+
     use MyTypes qw< FullyQualifiedDomainName >;
 
     type_subtest FullyQualifiedDomainName, sub {
@@ -553,6 +648,15 @@ version v0.90.1
                 nonprod3-foobar-me          nonprod3-foobar-me.ourdomain.com
             >,
         );
+        should_sort_into(
+            $type,
+            [qw< ftp001-prod3 ftp001-prod3.ourdomain.com prod-ask-me.ourdomain.com >],
+        );
+
+        like $type->get_message(undef), qr<Must be a valid FQDN>, 'error message is correct';
+        like $type->validate_explain(undef), [
+            qr<Undef did not pass type constraint>,
+        ], 'deep explanation is correct';
     };
 
     done_testing;
@@ -586,6 +690,10 @@ type within the code.
 
 These functions are most useful wrapped inside of a L</type_subtest> coderef.
 
+Note that most of these checks will run through C<get_message> and C<validate_explain> calls to
+confirm the coderefs don't die.  If you need to validate the error messages themselves, consider
+using checks similar to the ones in the L</SYNOPSIS>.
+
 =head3 should_pass_initially
 
     should_pass_initially($type, @values);
@@ -635,6 +743,20 @@ ref values as the "key".)
 The original value should not pass initial checks, as it would not be coerced in most use cases.
 These would be considered test failures.
 
+=head3 should_sort_into
+
+    should_sort_into($type, @sorted_arrayrefs);
+
+Creates a L<buffered subtest|Test2::Tools::Subtest/BUFFERED> that confirms the type will sort
+into the expected lists given.  The input list is a shuffled version of the sorted list.
+
+Because this introduces some non-deterministic behavior to the test, it will run through 100 cycles
+of shuffling and sorting to confirm the results.  A good sorter should always return a
+deterministic result for a given list, with enough fallbacks to account for every unique case.
+Any failure will immediate stop the loop and return both the shuffled input and output list in the
+failure output, so that you can temporarily test in a more deterministic manner, as you debug the
+fault.
+
 =head1 TROUBLESHOOTING
 
 =head2 Test name output
